==> component-loader-1.0.0.tgz: package/src/ComponentDependency.ts <==
import { ERR_MESSAGE } from "./const.ts";
import { LoadService } from "./LoadService.tsx";
import type { ComponentPackageItemDSL } from "@lowcode-set-up-platform/dsl";

/**
 * 主要管理组件的版本管理
 */
export interface ComponentVersion {
  name: string;
  libName: string;
  version: string;
  component: JSX.Element
}

export interface LoadRemoteComponentDependency {
  scriptsList: string[];
  styleList: string[];
  libName: string;
  lifecycle?: Lifecycle;
  content?: () => Record<any, any>
}



interface Lifecycle {
  onSuccess?: (...com: any) => any;
  onFailure?: (e: Error) => void;
}

interface Options {
  cross: boolean
}
/**
 * 通过代理模式管理页面所有组价的版本，每一个组件名映射多个版本，渲染组件时指定组件名和版本
 */
export class ComponentDependency {

  public componentMap: Map<string, Record<string, ComponentVersion>>

  public hostWindow: Window;
  public options: Options;

  constructor(hostWindow: Window, options: Options = { cross: true}) {
    this.componentMap = new Map()
    this.hostWindow = hostWindow || window;
    this.options = options;
  }

  /**
   * 读取组件
   * @param name
   * @param version
   */
  public async getComponent(name: string, version: string) {
    if (!version || !name) {
      return () => null;
    }
    const components = this.componentMap.get(name);
    return components?.[version]?.component;

  }

  /**
   * 设置组价
   * @param name
   * @param version
   * @param libName
   * @param component
   */
  public async setComponent(name: string, version: string, libName: string, component: JSX.Element) {
    if (this.componentMap.has(name)) {
      const components = this.componentMap.get(name) || {};
      if (!components[version]) {
        components[version] = {
          name,
          libName,
          component,
          version
        }
        this.componentMap.set(name, components);
      }
    }else {
      this.componentMap.set(name, {
        [version]: {
          name,
          libName,
          component,
          version
        }
      });
    }
  }

  /**
   * 删除组件
   * @param name
   * @param version
   * @param isAll
   */
  public deleteComponent(name: string, version: string, isAll?: boolean) {
    if(isAll === true) {
      return this.componentMap.delete(name);
    }
    const components = this.componentMap.get(name) || {}
    components[version] && delete components[version];
    this.componentMap.set(name, components);
  }

  /**
   * 从远程加载资源
   * @param name
   * @param version
   * @param options
   */
  public loadRemoteComponent(name: string, version: string, options: LoadRemoteComponentDependency): Promise<{
    scriptDom: HTMLScriptElement[];
    styleDom: HTMLStyleElement[];
    component: JSX.Element;
  }> {
    return new Promise(async (resolve, reject) => {
      if (!options.scriptsList || options.scriptsList.length === 0 || !options.libName) {
        const e = new Error(ERR_MESSAGE.Dependency_Empty)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
      let content = options.content;
      if (!content) {
        content = () => this.hostWindow
      }
      const com = await this.getComponent(name, version);
      if(com) {
        const e = new Error(ERR_MESSAGE.Component_Is_Ready)
        options?.lifecycle?.onFailure?.(e);
        return reject(new Error(ERR_MESSAGE.Component_Is_Ready));
      }
      try {
        const { scriptsList = [], styleList = []} = options;
        const scriptDom: HTMLScriptElement[] = [];
        const styleDom: HTMLStyleElement[] = [];

        const loadService = new LoadService({
          enableSandbox: true,
          window: this.hostWindow,
          cross: this.options.cross
        })
        const stylePromises = [];
        const jsPromises = [];
        for (let i = 0; i < styleList?.length; i ++) {
          stylePromises.push(loadService.importStyle(styleList[i]))
        }
        for (let i = 0; i < scriptsList?.length; i ++) {
          jsPromises.push(loadService.importScript(scriptsList[i]))
        }

        const styleResult = await Promise.all(stylePromises);
        const scriptResult = await Promise.all(jsPromises);
        for (let j = 0; j < styleResult.length; j ++) {
          const style: HTMLStyleElement = styleResult[j];
          styleDom.push(style)
          this.hostWindow.document.head.appendChild(style)
        }
        for (let j = 0; j < scriptResult.length; j ++) {
          const js: HTMLScriptElement = scriptResult[j];
          scriptDom.push(js)
        }
        const hostContent = content();

        const comp = hostContent[options.libName] as JSX.Element
        // 同步组件到组件依赖中
        await this.setComponent(name, version, options.libName, comp)
        resolve({
          scriptDom,
          styleDom,
          component: comp
        })
        void options?.lifecycle?.onSuccess?.(comp);
      }catch (e: any) {
        console.log(e)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
    })
  }

  /**
   * 解析组件
   */
  public async parseComponentsMap(componentsMap: ComponentPackageItemDSL[]): Promise<Record<string, any>> {
    return new Promise(async (resolve) => {
      const componentMap: Record<string, any> = {};
      for (let i = 0; i < componentsMap.length; i ++) {
        const { componentName, version, main, style, destructuring, exportName } = componentsMap[i];
        // @ts-ignore
        const content = () => destructuring? this.hostWindow[exportName]: this.hostWindow
        let comInfo = await this.getComponent(componentName, version);
        if (!comInfo) {
          const styleList: string[] = style? [style]: [];
          const scriptsList: string[] = main? [main]: [];
          await this.loadRemoteComponent(componentName, version, {
            styleList: styleList,
            scriptsList: scriptsList,
            libName: componentName,
            lifecycle: {
              onSuccess: (_c: any) => {},
              onFailure: (_e: Error) => {}
            },
            content
          })
        }
        comInfo = await this.getComponent(componentName, version);
        componentMap[componentName] = comInfo;
      }
      resolve(componentMap)
    })
  }

  /**
   * 从远程加载资源
   * @param options
   */
  public loadRemoteLib(options: LoadRemoteComponentDependency): Promise<{
    scriptDom: HTMLScriptElement[];
    styleDom: HTMLStyleElement[];
  }> {
    return new Promise(async (resolve, reject) => {
      if (!options.scriptsList || options.scriptsList.length === 0 || !options.libName) {
        const e = new Error(ERR_MESSAGE.Dependency_Empty)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
      try {
        const { scriptsList = [], styleList = []} = options;
        const scriptDom: HTMLScriptElement[] = [];
        const styleDom: HTMLStyleElement[] = [];

        const loadService = new LoadService({
          enableSandbox: true,
          window: this.hostWindow,
          cross: this.options.cross
        })
        const stylePromises = [];
        const jsPromises = [];
        for (let i = 0; i < styleList?.length; i ++) {
          stylePromises.push(loadService.importStyle(styleList[i]))
        }
        for (let i = 0; i < scriptsList?.length; i ++) {
          jsPromises.push(loadService.importScript(scriptsList[i]))
        }

        const styleResult = await Promise.all(stylePromises);
        const scriptResult = await Promise.all(jsPromises);
        for (let j = 0; j < styleResult.length; j ++) {
          const style: HTMLStyleElement = styleResult[j];
          styleDom.push(style)
          this.hostWindow.document.head.appendChild(style)
        }
        for (let j = 0; j < scriptResult.length; j ++) {
          const js: HTMLScriptElement = scriptResult[j];
          scriptDom.push(js)
        }
        resolve({
          scriptDom,
          styleDom,
        })
        void options?.lifecycle?.onSuccess?.();
      }catch (e: any) {
        console.log(e)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
    })
  }

  /**
   * 解析依赖包
   */
  public async parsePackagesMap(componentsMap: ComponentPackageItemDSL[]): Promise<boolean> {
    return new Promise(async (resolve) => {
      for (let i = 0; i < componentsMap.length; i ++) {
        // exportName 是暴露出的包，如果是子包的话 需要从 componentName 下读取
        const { exportName: componentName, main, style} = componentsMap[i];
        // 此处判断下依赖库的版本
        // @ts-ignore
        if(!this.hostWindow[componentName]) {
          const styleList: string[] = style? [style]: [];
          const scriptsList: string[] = main? [main]: [];
          await this.loadRemoteLib({
            styleList: styleList,
            scriptsList: scriptsList,
            libName: componentName,
            lifecycle: {
              onSuccess: (_c: any) => {},
              onFailure: (_e: Error) => {}
            }
          })
        }
      }
      resolve(true)
    })
  }

}

==> component-loader-2.1.6.tgz: package/src/ComponentDependency.ts <==
import {ERR_MESSAGE, NPM_CONFIG} from "./const.ts";
import {LoadService} from "./LoadService.tsx";
import type {ComponentPackageItemDSL} from "@lowcode-set-up-platform/dsl";

/**
 * 主要管理组件的版本管理
 */
export interface ComponentVersion {
  name: string;
  libName: string;
  version: string;
  component: JSX.Element
}

export interface LoadRemoteComponentDependency {
  scriptsList: string[];
  styleList: string[];
  libName: string;
  lifecycle?: Lifecycle;
  content?: () => Record<any, any>
}



interface Lifecycle {
  onSuccess?: (...com: any) => any;
  onFailure?: (e: Error) => void;
}

interface Options {
  cross: boolean
}
/**
 * 通过代理模式管理页面所有组价的版本，每一个组件名映射多个版本，渲染组件时指定组件名和版本
 */
export class ComponentDependency {

  public sandboxVersion = false;

  public componentMap: Map<string, Record<string, ComponentVersion>>

  public hostWindow: Window;
  public options: Options;

  constructor(hostWindow: Window, options: Options = { cross: true}) {
    this.componentMap = new Map()
    this.hostWindow = hostWindow || window;
    this.options = options;
  }

  public formatSourceUrl(packageName: string, version: string, isLib = false): {js: string[]; style: string[]} {
    const js: string[] = [];
    const style: string[] = [];

    js.push(`${NPM_CONFIG.DEFAULT_CDN_DOMAIN}${NPM_CONFIG.DEFAULT_CDN_DOMAIN_BASE_URL}/${packageName}/${version}/${NPM_CONFIG.materialSourceType}/${NPM_CONFIG.scriptFileName(packageName)}`)

    if (!isLib) {
      style.push(`${NPM_CONFIG.DEFAULT_CDN_DOMAIN}${NPM_CONFIG.DEFAULT_CDN_DOMAIN_BASE_URL}/${packageName}/${version}/${NPM_CONFIG.materialSourceType}/${NPM_CONFIG.styleFileName()}`)
    }
    return {
      js,
      style
    }

  }

  /**
   * 读取组件
   * @param name
   * @param version
   */
  public async getComponent(name: string, version: string) {
    if (!version || !name) {
      return () => null;
    }
    const components = this.componentMap.get(name);
    return components?.[version]?.component;

  }

  /**
   * 设置组价
   * @param name
   * @param version
   * @param libName
   * @param component
   */
  public async setComponent(name: string, version: string, libName: string, component: JSX.Element | any) {
    if (this.componentMap.has(name)) {
      const components = this.componentMap.get(name) || {};
      if (!components[version]) {
        components[version] = {
          name,
          libName,
          component,
          version
        }
        this.componentMap.set(name, components);
      }
    }else {
      this.componentMap.set(name, {
        [version]: {
          name,
          libName,
          component,
          version
        }
      });
    }
  }

  /**
   * 删除组件
   * @param name
   * @param version
   * @param isAll
   */
  public deleteComponent(name: string, version: string, isAll?: boolean) {
    if(isAll === true) {
      return this.componentMap.delete(name);
    }
    const components = this.componentMap.get(name) || {}
    components[version] && delete components[version];
    this.componentMap.set(name, components);
  }

  /**
   * 从远程加载资源
   * @param name
   * @param version
   * @param options
   */
  public loadRemoteComponent(name: string, version: string, options: LoadRemoteComponentDependency): Promise<{
    scriptDom: HTMLScriptElement[];
    styleDom: HTMLStyleElement[];
    component: JSX.Element;
  }> {
    return new Promise(async (resolve, reject) => {
      if (!options.scriptsList || options.scriptsList.length === 0 || !options.libName) {
        const e = new Error(ERR_MESSAGE.Dependency_Empty)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
      let content = options.content;
      if (!content) {
        content = () => this.hostWindow
      }
      const com = await this.getComponent(name, version);
      if(com) {
        const e = new Error(ERR_MESSAGE.Component_Is_Ready)
        options?.lifecycle?.onFailure?.(e);
        return reject(new Error(ERR_MESSAGE.Component_Is_Ready));
      }
      try {
        const { scriptsList = [], styleList = []} = options;
        const scriptDom: HTMLScriptElement[] = [];
        const styleDom: HTMLStyleElement[] = [];

        const loadService = new LoadService({
          enableSandbox: true,
          window: this.hostWindow,
          cross: this.options.cross
        })
        const stylePromises = [];
        const jsPromises = [];
        for (let i = 0; i < styleList?.length; i ++) {
          stylePromises.push(loadService.importStyle(styleList[i]))
        }
        for (let i = 0; i < scriptsList?.length; i ++) {
          jsPromises.push(loadService.importScript(scriptsList[i]))
        }

        const styleResult = await Promise.all(stylePromises);
        const scriptResult = await Promise.all(jsPromises);
        for (let j = 0; j < styleResult.length; j ++) {
          const style: HTMLStyleElement = styleResult[j];
          styleDom.push(style)
          this.hostWindow.document.head.appendChild(style)
        }
        for (let j = 0; j < scriptResult.length; j ++) {
          const js: HTMLScriptElement = scriptResult[j];
          scriptDom.push(js)
        }
        const hostContent = content();

        const comp = hostContent[options.libName] as JSX.Element
        // 同步组件到组件依赖中
        await this.setComponent(name, version, options.libName, comp)
        resolve({
          scriptDom,
          styleDom,
          component: comp
        })
        void options?.lifecycle?.onSuccess?.(comp);
      }catch (e: any) {
        console.log(e)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
    })
  }

  /**
   * 解析组件
   */
  public async parseComponentsMap(componentsMap: ComponentPackageItemDSL[]): Promise<Record<string, any>> {
    return new Promise(async (resolve) => {
      const componentMap: Record<string, any> = {};
      for (let i = 0; i < componentsMap.length; i ++) {
        const { componentName, version, destructuring, exportName,  } = componentsMap[i];
        // @ts-ignore
        const content = () => destructuring? this.hostWindow[exportName]: this.hostWindow
        let comInfo = await this.getComponent(componentsMap[i].package, version);
        if (!comInfo) {
          // 开启版本隔离或者搭建过程中， 异步加载
          // @ts-ignore
          if (this.sandboxVersion || window.__building__ || !content()[exportName]) {
            const { js, style} = this.formatSourceUrl(componentsMap[i].package, componentsMap[i].version, false)
            await this.loadRemoteComponent(componentsMap[i].package, version, {
              styleList: style,
              scriptsList: js,
              libName: componentName,
              lifecycle: {
                onSuccess: (_c: any) => {},
                onFailure: (_e: Error) => {}
              },
              content
            })
          }
          else {
            const Comp = content()[exportName]
            await this.setComponent(componentsMap[i].package, version, exportName, Comp!)
          }

        }
        comInfo = await this.getComponent(componentsMap[i].package, version);
        componentMap[componentName] = comInfo;
      }
      resolve(componentMap)
    })
  }

  /**
   * 从远程加载资源
   * @param options
   */
  public loadRemoteLib(options: LoadRemoteComponentDependency): Promise<{
    scriptDom: HTMLScriptElement[];
    styleDom: HTMLStyleElement[];
  }> {
    return new Promise(async (resolve, reject) => {
      if (!options.scriptsList || options.scriptsList.length === 0 || !options.libName) {
        const e = new Error(ERR_MESSAGE.Dependency_Empty)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
      try {
        const { scriptsList = [], styleList = []} = options;
        const scriptDom: HTMLScriptElement[] = [];
        const styleDom: HTMLStyleElement[] = [];

        const loadService = new LoadService({
          enableSandbox: true,
          window: this.hostWindow,
          cross: this.options.cross
        })
        const stylePromises = [];
        const jsPromises = [];
        for (let i = 0; i < styleList?.length; i ++) {
          stylePromises.push(loadService.importStyle(styleList[i]))
        }
        for (let i = 0; i < scriptsList?.length; i ++) {
          jsPromises.push(loadService.importScript(scriptsList[i]))
        }

        const styleResult = await Promise.all(stylePromises);
        const scriptResult = await Promise.all(jsPromises);
        for (let j = 0; j < styleResult.length; j ++) {
          const style: HTMLStyleElement = styleResult[j];
          styleDom.push(style)
          this.hostWindow.document.head.appendChild(style)
        }
        for (let j = 0; j < scriptResult.length; j ++) {
          const js: HTMLScriptElement = scriptResult[j];
          scriptDom.push(js)
        }
        resolve({
          scriptDom,
          styleDom,
        })
        void options?.lifecycle?.onSuccess?.();
      }catch (e: any) {
        console.log(e)
        options?.lifecycle?.onFailure?.(e);
        return reject(e)
      }
    })
  }

  /**
   * 解析依赖包
   */
  public async parsePackagesMap(componentsMap: ComponentPackageItemDSL[]): Promise<boolean> {
    return new Promise(async (resolve) => {
      for (let i = 0; i < componentsMap.length; i ++) {
        // exportName 是暴露出的包，如果是子包的话 需要从 componentName 下读取
        const { exportName: componentName} = componentsMap[i];
        // 此处判断下依赖库的版本
        // @ts-ignore
        if(!this.hostWindow[componentName]) {
          const { js, style} = this.formatSourceUrl(componentsMap[i].package, componentsMap[i].version, true)
          await this.loadRemoteLib({
            styleList: style,
            scriptsList: js,
            libName: componentName,
            lifecycle: {
              onSuccess: (_c: any) => {},
              onFailure: (_e: Error) => {}
            }
          })
        }
      }
      resolve(true)
    })
  }

}
